###
Recipe.run(function(rctx) {
      ...
  });
###

sputs = (str) ->
   CHPrinter.print_(str)

tostr = (x) ->
   if x == null
      return ""
   if typeof x == 'undefined'
      return ""
   if x == 0
      return ""
   return "" + x

tonsstr = (x) ->
   if x == null || x == ""
      return NSString.string()
   return NSString.stringWithString_(str)

nsstr_to_str = (x) ->
   if x.length() == 0
      return ""
   if x == null
      return ""
   if x == undefined || typeof(x) == "undefined"
      return ""
   return "" + x

indentation = (str) ->
   extent = 0
   ended = false
   i = 0
   l = str.length
   while i < l
      c = str.charAt(i)
      if c == " " || c == "\t"
         if ended == false
            extent += 1
      else
         break
      i++
      
   if extent == 0 || extent == "0"
      return ""
   return str.substr(0, extent)

String.prototype.spliceSubstr = (part, loc, len) ->
   this.substr(0, loc) + part + this.substr(loc + len, this.length - loc - len)

#- Context
#- A list of 
Context =
   textView: () -> _context.valueForKey_("textView")
   textViewController: () -> _context.valueForKey_("textViewController")
   document: () -> _context.valueForKey_("document")
   project: () -> _context.valueForKey_("project")
   # TODO: environment(). Get an environment dictionary
   tabInfo: () ->
     ti = Context.document().tabInfo()
     return if ti then {
        'tabSize': ti.tabSize(),
        'softTabs': ti.useSoftTabs(),
     } else null
   context: () -> nsstr_to_str(Context.textViewController().selectionContext().basicContext())

#- Language
Language =
   metadata: (lid) ->
      _spldb
   withIdentifier: (lid) ->
      { "metadata": _spldb[lid] }
   withExtension: (ext) ->
      for lid of _spldb
         if _spldb[lid]["extensions"] == ext || _.include(_spldb[lid]["extensions"], ext)
            return Language.withIdentifier(lid)
      return null
   current: ->
      Language.withIdentifier(nsstr_to_str(Context.textViewController().topmostScopeLanguageName()))

selector_matches = (sel, ctx) -> !!(CHSelector.compileString_(sel).matchesBasicContext_(ctx))

#- Recipe
#- A Recipe is list of instructions you can take to modify
class Recipe
   @translateLineInfo: (lf) ->
      return if lf? then {
         'contentRange': lf.valueForKey_("contentRange").rangeValue(),
         'conventionalRange': lf.valueForKey_("conventionalRange").rangeValue(),

         'inclusiveStartLocation': lf.valueForKey_("inclusiveStartLocation").unsignedIntegerValue(),
         'contentStartLocation': lf.valueForKey_("contentStartLocation").unsignedIntegerValue(),
         'contentEndLocation': lf.valueForKey_("contentEndLocation").unsignedIntegerValue(),
         'inclusiveEndLocation': lf.valueForKey_("inclusiveEndLocation").unsignedIntegerValue(),

         'startOfFirstLine': lf.valueForKey_("startOfFirstLine").unsignedIntegerValue(),
         'startOfLastLine': lf.valueForKey_("startOfLastLine").unsignedIntegerValue(),
         'offset': lf.valueForKey_("offset").unsignedIntegerValue(),

         'index': lf.valueForKey_("index").unsignedIntegerValue(),
      } else null
      
   @run: (rfunc) ->
      recipe = new Recipe()
      recipe.r = CHRecipe.alloc().initWithTextView_document_(Context.textView(), Context.document())
      
      # Run the recipe
      recipe.r.applyPreprocess()
      rfunc(recipe)
      recipe.r.applyPostprocess()
      return
   
   ##- length: length() -> Int
   ##- Returns the current length of the recipe's working string.
   length: ->
      @r.length()
   
   ##- text: text() -> String
   ##- Returns the current working string
   text: ->
      nsstr_to_str @r.text()
   
   ##- textInRange: textInRange(range) -> String
   ##- Returns a substring of the recipe's working string
   textInRange: (range) ->
      nsstr_to_str @r.textInRange_(range)
   
   ##- rangeOfLinesInRange: rangeOfLinesInRange(range) -> Range
   ##- When given a character range in the working string, returns a character range of those lines.
   rangeOfLinesInRange: (range) ->
      @r.rangeOfLineInRange_(range)
   contentRangeOfLinesInRange: (range) ->
      @r.contentRangeOfLinesInRange_(range)
   
   # Range -> Range
   lineIndexesForCharacterRange: (range) ->
   
   # Range -> Range
   characterRangeForLineIndexes: (range) ->
   
   # Range -> Marker[]
   lineMarkersForCharacterRange: (range) ->
   
   # foreachLine(each_f: LineInfo -> String, optional_range) -> Range
   foreachLine: (each_f, optional_range) ->
      optional_range ?= NSMakeRange(0, @r.length())
      re = CHRecipeEnumerator.alloc().init()
      re.setUp_inRange_(@r, optional_range)
      
      workstr = @.textInRange(optional_range)
      modifiedWorkstr = false
      
      offset = 0
      while 1
         lineInfo = re.nextObject()
         lf = Recipe.translateLineInfo(lineInfo)
         if !(lf?)
            break
         l = each_f(lf)
         if l? || l == false
            if l == false
               workstr = workstr.spliceSubstr("", offset + lf.contentStartLocation - lf.startOfFirstLine, lf.inclusiveEndLocation - lf.contentStartLocation)
               modifiedWorkstr = true
               offset -= (lf.inclusiveEndLocation - lf.contentStartLocation)
            else
               workstr = workstr.spliceSubstr(l, offset + lf.contentStartLocation - lf.startOfFirstLine, lf.contentEndLocation - lf.contentStartLocation)
               modifiedWorkstr = true
               offset += (l).length - (lf.contentEndLocation - lf.contentStartLocation)
         
      if modifiedWorkstr
         @.replaceTextInRange(optional_range, workstr)
      return offset
   
   ##- replaceTextInRange: replaceTextInRange(range, replacement: String) -> Bool
   replaceTextInRange: (range, replacement) ->
      @r.replaceTextInRange_with_(range, replacement)
   
   ##- deleteTextInRange: deleteTextInRange(range, replacement: String [, recordUndo: Bool]) -> Bool
   deleteTextInRange: (range, opt_recordUndo) ->
      @replaceTextInRange(range, "", opt_recordUndo)
   
   ##- insertTextAtLocation: insertTextAtLocation(location: Int, insertText: String [, recordUndo: Bool]) -> Bool
   insertTextAtLocation: (location, new_text, opt_recordUndo) ->
      @replaceTextInRange(NSMakeRange(location, 0), new_text, opt_recordUndo)
   
Recipe.prototype.__defineGetter__("selection", ->
   @r.selection()
)
Recipe.prototype.__defineSetter__("selection", (newSel) ->
   @r.setSelection_(newSel)
)

Recipe.prototype.__defineGetter__("jumpToShowSelection", ->
   @r.jumpToShowSelection() 
)
Recipe.prototype.__defineSetter__("jumpToShowSelection", (b) ->
   @r.setJumpToShowSelection_(b)
)

Recipe.prototype.__defineGetter__("showYellowIndicator", ->
   @r.showYellowIndicator() 
)
Recipe.prototype.__defineSetter__("showYellowIndicator", (b) ->
   @r.setShowYellowIndicator(b)
)
