/*
Recipe.run(function(rctx) {
      ...
  });
*/
var Context, Language, Recipe, indentation, nsstr_to_str, selector_matches, sputs, tonsstr, tostr;
sputs = function(str) {
  return CHPrinter.print_(str);
};
tostr = function(x) {
  if (x === null) {
    return "";
  }
  if (typeof x === 'undefined') {
    return "";
  }
  if (x === 0) {
    return "";
  }
  return "" + x;
};
tonsstr = function(x) {
  if (x === null || x === "") {
    return NSString.string();
  }
  return NSString.stringWithString_(str);
};
nsstr_to_str = function(x) {
  if (x.length() === 0) {
    return "";
  }
  if (x === null) {
    return "";
  }
  if (x === void 0 || typeof x === "undefined") {
    return "";
  }
  return "" + x;
};
indentation = function(str) {
  var c, ended, extent, i, l;
  extent = 0;
  ended = false;
  i = 0;
  l = str.length;
  while (i < l) {
    c = str.charAt(i);
    if (c === " " || c === "\t") {
      if (ended === false) {
        extent += 1;
      }
    } else {
      break;
    }
    i++;
  }
  if (extent === 0 || extent === "0") {
    return "";
  }
  return str.substr(0, extent);
};
String.prototype.spliceSubstr = function(part, loc, len) {
  return this.substr(0, loc) + part + this.substr(loc + len, this.length - loc - len);
};
Context = {
  textView: function() {
    return _context.valueForKey_("textView");
  },
  textViewController: function() {
    return _context.valueForKey_("textViewController");
  },
  document: function() {
    return _context.valueForKey_("document");
  },
  project: function() {
    return _context.valueForKey_("project");
  },
  tabInfo: function() {
    var ti;
    ti = Context.document().tabInfo();
    if (ti) {
      return {
        'tabSize': ti.tabSize(),
        'softTabs': ti.useSoftTabs()
      };
    } else {
      return null;
    }
  },
  context: function() {
    return nsstr_to_str(Context.textViewController().selectionContext().basicContext());
  }
};
Language = {
  metadata: function(lid) {
    return _spldb;
  },
  withIdentifier: function(lid) {
    return {
      "metadata": _spldb[lid]
    };
  },
  withExtension: function(ext) {
    var lid;
    for (lid in _spldb) {
      if (_spldb[lid]["extensions"] === ext || _.include(_spldb[lid]["extensions"], ext)) {
        return Language.withIdentifier(lid);
      }
    }
    return null;
  },
  current: function() {
    return Language.withIdentifier(nsstr_to_str(Context.textViewController().topmostScopeLanguageName()));
  }
};
selector_matches = function(sel, ctx) {
  return !!(CHSelector.compileString_(sel).matchesBasicContext_(ctx));
};
Recipe = (function() {
  function Recipe() {}
  Recipe.translateLineInfo = function(lf) {
    if (lf != null) {
      return {
        'contentRange': lf.valueForKey_("contentRange").rangeValue(),
        'conventionalRange': lf.valueForKey_("conventionalRange").rangeValue(),
        'inclusiveStartLocation': lf.valueForKey_("inclusiveStartLocation").unsignedIntegerValue(),
        'contentStartLocation': lf.valueForKey_("contentStartLocation").unsignedIntegerValue(),
        'contentEndLocation': lf.valueForKey_("contentEndLocation").unsignedIntegerValue(),
        'inclusiveEndLocation': lf.valueForKey_("inclusiveEndLocation").unsignedIntegerValue(),
        'startOfFirstLine': lf.valueForKey_("startOfFirstLine").unsignedIntegerValue(),
        'startOfLastLine': lf.valueForKey_("startOfLastLine").unsignedIntegerValue(),
        'offset': lf.valueForKey_("offset").unsignedIntegerValue(),
        'index': lf.valueForKey_("index").unsignedIntegerValue()
      };
    } else {
      return null;
    }
  };
  Recipe.run = function(rfunc) {
    var recipe;
    recipe = new Recipe();
    recipe.r = CHRecipe.alloc().initWithTextView_document_(Context.textView(), Context.document());
    recipe.r.applyPreprocess();
    rfunc(recipe);
    recipe.r.applyPostprocess();
  };
  Recipe.prototype.length = function() {
    return this.r.length();
  };
  Recipe.prototype.text = function() {
    return nsstr_to_str(this.r.text());
  };
  Recipe.prototype.textInRange = function(range) {
    return nsstr_to_str(this.r.textInRange_(range));
  };
  Recipe.prototype.rangeOfLinesInRange = function(range) {
    return this.r.rangeOfLineInRange_(range);
  };
  Recipe.prototype.contentRangeOfLinesInRange = function(range) {
    return this.r.contentRangeOfLinesInRange_(range);
  };
  Recipe.prototype.lineIndexesForCharacterRange = function(range) {};
  Recipe.prototype.characterRangeForLineIndexes = function(range) {};
  Recipe.prototype.lineMarkersForCharacterRange = function(range) {};
  Recipe.prototype.foreachLine = function(each_f, optional_range) {
    var l, lf, lineInfo, modifiedWorkstr, offset, re, workstr;
    if (optional_range == null) {
      optional_range = NSMakeRange(0, this.r.length());
    }
    re = CHRecipeEnumerator.alloc().init();
    re.setUp_inRange_(this.r, optional_range);
    workstr = this.textInRange(optional_range);
    modifiedWorkstr = false;
    offset = 0;
    while (1) {
      lineInfo = re.nextObject();
      lf = Recipe.translateLineInfo(lineInfo);
      if (!(lf != null)) {
        break;
      }
      l = each_f(lf);
      if ((l != null) || l === false) {
        if (l === false) {
          workstr = workstr.spliceSubstr("", offset + lf.contentStartLocation - lf.startOfFirstLine, lf.inclusiveEndLocation - lf.contentStartLocation);
          modifiedWorkstr = true;
          offset -= lf.inclusiveEndLocation - lf.contentStartLocation;
        } else {
          workstr = workstr.spliceSubstr(l, offset + lf.contentStartLocation - lf.startOfFirstLine, lf.contentEndLocation - lf.contentStartLocation);
          modifiedWorkstr = true;
          offset += l.length - (lf.contentEndLocation - lf.contentStartLocation);
        }
      }
    }
    if (modifiedWorkstr) {
      this.replaceTextInRange(optional_range, workstr);
    }
    return offset;
  };
  Recipe.prototype.replaceTextInRange = function(range, replacement) {
    return this.r.replaceTextInRange_with_(range, replacement);
  };
  Recipe.prototype.deleteTextInRange = function(range, opt_recordUndo) {
    return this.replaceTextInRange(range, "", opt_recordUndo);
  };
  Recipe.prototype.insertTextAtLocation = function(location, new_text, opt_recordUndo) {
    return this.replaceTextInRange(NSMakeRange(location, 0), new_text, opt_recordUndo);
  };
  return Recipe;
})();
Recipe.prototype.__defineGetter__("selection", function() {
  return this.r.selection();
});
Recipe.prototype.__defineSetter__("selection", function(newSel) {
  return this.r.setSelection_(newSel);
});
Recipe.prototype.__defineGetter__("jumpToShowSelection", function() {
  return this.r.jumpToShowSelection();
});
Recipe.prototype.__defineSetter__("jumpToShowSelection", function(b) {
  return this.r.setJumpToShowSelection_(b);
});
Recipe.prototype.__defineGetter__("showYellowIndicator", function() {
  return this.r.showYellowIndicator();
});
Recipe.prototype.__defineSetter__("showYellowIndicator", function(b) {
  return this.r.setShowYellowIndicator(b);
});